// -------------------------------------------------------------------------
// PARAMETERS:
// -------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (DX9)
float4x4 tP: PROJECTION;   //projection matrix as set via Renderer (DX9)
float4x4 tWVP: WORLDVIEWPROJECTION;

//texture
texture Tex <string uiname="Texture";>;
texture Tex2 <string uiname="Texture2";>;

sampler Samp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex);          //apply a texture to the sampler
  MipFilter = LINEAR; //sampler states
  MinFilter = LINEAR;
  MagFilter = LINEAR;
};

sampler Displacement = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex2);          //apply a texture to the sampler
    MipFilter = POINT;         //sampler states
    MinFilter = POINT;
    MagFilter = POINT;
};

//texture transformation marked with semantic TEXTUREMATRIX to achieve symmetric transformations
float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;

//the data structure: "vertexshader to pixelshader"
//used as output data with the VS function
//and as input data with the PS function
struct vs3ps
{
    float4 Pos   : POSITION;
    float2 TexCood : TEXCOORD0;
    float zDisplacement : TEXCOORD1;
};
// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------

vs3ps VS(
    float4 Pos : POSITION,
    float4 normal : Normal,
    float2 TexCood : TEXCOORD0
    )
{
    //inititalize all fields of output struct with 0
    vs3ps Out = (vs3ps)0;

   float displaceVal = tex2Dlod(Displacement, float4(TexCood.x, TexCood.y,  0,0));

   float4 displacedPos;
   displacedPos = Pos;
   float zDisplacement = ( displaceVal);
   displacedPos.z = Pos.z + displaceVal;

   Out.zDisplacement = displaceVal;
   //transform position
   Out.Pos = mul(displacedPos, tWVP);
   Out.TexCood = mul(TexCood, tTex);

   return Out;
}

// -------------------------------------------------------------------------
// PIXELSHADERS:
// -------------------------------------------------------------------------

float4 PX(vs3ps In): COLOR
{

    //texture lookup to access the pixels of the texture
    float4 col = tex2D(Samp, In.TexCood);
    float4 dis = float4( In.zDisplacement,0,0,1);
    return col+dis;
}
// -------------------------------------------------------------------------
// TECHNIQUES:
// -------------------------------------------------------------------------

technique DisplacementMapping_Solid
{
    pass P0
    {

       VertexShader = compile vs_3_0 VS();
       PixelShader  = compile ps_3_0 PX();

    }
}

technique DisplacementMapping_WireFrame
{
    pass P0
    {
       fillmode = wireframe;
       VertexShader = compile vs_3_0 VS();
       PixelShader  = compile ps_3_0 PX();

    }
}
